@@ -10,7 +10,7 @@ class TestGemCommandsSetupCommand < Gem::TestCase
   if File.exist?(bundler_gemspec)
     BUNDLER_VERS = File.read(bundler_gemspec).match(/VERSION = "(#{Gem::Version::VERSION_PATTERN})"/)[1]
   else
-    BUNDLER_VERS = "1.16.2"
+    BUNDLER_VERS = "1.16.2".freeze
   end
 
   def setup
@@ -38,6 +38,7 @@ class TestGemCommandsSetupCommand < Gem::TestCase
     FileUtils.mkdir_p 'default/gems'
 
     gemspec = Gem::Specification.new
+    gemspec.author = "Us"
     gemspec.name = "bundler"
     gemspec.version = BUNDLER_VERS
     gemspec.bindir = "exe"
@@ -66,7 +67,7 @@ class TestGemCommandsSetupCommand < Gem::TestCase
     FileUtils.mkdir_p 'default/gems/bundler-audit-1.0.0'
   end
 
-  def gem_install name
+  def gem_install(name)
     gem = util_spec name do |s|
       s.executables = [name]
       s.files = %W[bin/#{name}]
@@ -103,6 +104,40 @@ class TestGemCommandsSetupCommand < Gem::TestCase
     assert_equal "I changed it!\n", File.read(gem_bin_path)
   end
 
+  def test_env_shebang_flag
+    gem_bin_path = gem_install 'a'
+    write_file gem_bin_path do |io|
+      io.puts 'I changed it!'
+    end
+
+    @cmd.options[:document] = []
+    @cmd.options[:env_shebang] = true
+    @cmd.execute
+
+    gem_exec = sprintf Gem.default_exec_format, 'gem'
+    default_gem_bin_path = File.join @install_dir, 'bin', gem_exec
+    if Gem::USE_BUNDLER_FOR_GEMDEPS
+      bundle_exec = sprintf Gem.default_exec_format, 'bundle'
+      default_bundle_bin_path = File.join @install_dir, 'bin', bundle_exec
+    end
+
+    ruby_exec = sprintf Gem.default_exec_format, 'ruby'
+
+    if Gem.win_platform?
+      assert_match %r%\A#!\s*#{ruby_exec}%, File.read(default_gem_bin_path)
+      if Gem::USE_BUNDLER_FOR_GEMDEPS
+        assert_match %r%\A#!\s*#{ruby_exec}%, File.read(default_bundle_bin_path)
+      end
+      assert_match %r%\A#!\s*#{ruby_exec}%, File.read(gem_bin_path)
+    else
+      assert_match %r%\A#!/usr/bin/env #{ruby_exec}%, File.read(default_gem_bin_path)
+      if Gem::USE_BUNDLER_FOR_GEMDEPS
+        assert_match %r%\A#!/usr/bin/env #{ruby_exec}%, File.read(default_bundle_bin_path)
+      end
+      assert_match %r%\A#!/usr/bin/env #{ruby_exec}%, File.read(gem_bin_path)
+    end
+  end
+
   def test_pem_files_in
     assert_equal %w[rubygems/ssl_certs/rubygems.org/foo.pem],
                  @cmd.pem_files_in('lib').sort
@@ -132,16 +167,19 @@ class TestGemCommandsSetupCommand < Gem::TestCase
   def test_install_default_bundler_gem
     @cmd.extend FileUtils
 
-    @cmd.install_default_bundler_gem
+    bin_dir = File.join(@gemhome, 'bin')
+    @cmd.install_default_bundler_gem bin_dir
 
-    if Gem.win_platform?
-      bundler_spec = Gem::Specification.load("bundler/bundler.gemspec")
-      default_spec_path = File.join(Gem::Specification.default_specifications_dir, "#{bundler_spec.full_name}.gemspec")
-      spec = Gem::Specification.load(default_spec_path)
+    bundler_spec = Gem::Specification.load("bundler/bundler.gemspec")
+    default_spec_path = File.join(Gem::Specification.default_specifications_dir, "#{bundler_spec.full_name}.gemspec")
+    spec = Gem::Specification.load(default_spec_path)
 
-      spec.executables.each do |e|
-        assert_path_exists File.join(spec.bin_dir, "#{e}.bat")
+    spec.executables.each do |e|
+      if Gem.win_platform?
+        assert_path_exists File.join(bin_dir, "#{e}.bat")
       end
+
+      assert_path_exists File.join bin_dir, e
     end
 
     default_dir = Gem::Specification.default_specifications_dir
@@ -207,11 +245,8 @@ class TestGemCommandsSetupCommand < Gem::TestCase
   end
 
   def test_show_release_notes
-    @default_external = nil
-    if Object.const_defined? :Encoding
-      @default_external = @ui.outs.external_encoding
-      @ui.outs.set_encoding Encoding::US_ASCII
-    end
+    @default_external = @ui.outs.external_encoding
+    @ui.outs.set_encoding Encoding::US_ASCII
 
     @cmd.options[:previous_version] = Gem::Version.new '2.0.2'
 
@@ -256,7 +291,7 @@ class TestGemCommandsSetupCommand < Gem::TestCase
     EXPECTED
 
     output = @ui.output
-    output.force_encoding Encoding::UTF_8 if Object.const_defined? :Encoding
+    output.force_encoding Encoding::UTF_8
 
     assert_equal expected, output
   ensure
